//
//  INKImageLoader.m
//  Net Kit Sample
//
//  Created by iTech on 24.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "INKImageLoader.h"

#import "INKURLRequest.h"

@interface INKImageLoader ()

- (void)downloadImageFromLink:(NSString *)link forKey:(NSString *)key withOperationBlock:(INKImageLoaderOperationBlock)block important:(BOOL)isImportant;

@end

@implementation INKImageLoader
@synthesize requestsManager = requestsManager;
@synthesize requestsCachePolicy = _requestsCachePolicy;
@synthesize requestsTimeout = _requestsTimeout;


- (void)dealloc {
    [requestsManager release];
    [super dealloc];
}

- (id)init {
    self = [super init];
    if (self) {
        _requestsCachePolicy = NSURLRequestUseProtocolCachePolicy;
        _requestsTimeout = 60.0;
        requestsManager = [[INKRequestsManager alloc] init];
        requestsInfo = [[NSMutableDictionary alloc] init];
        [self setMaximumImageDownloads:5];
    }
    return self;
}

#pragma mark Accessors

- (NSInteger)maximumImagesDownload {
    return [requestsManager processingRequestsLimit];
}

- (void)setMaximumImageDownloads:(NSInteger)downloadsLimit {
    requestsManager.processingRequestsLimit = downloadsLimit;
}

- (float)downloadProgressOfImageForKey:(NSString *)key {
    if ([self isImageForKeyWaitingInQueue:key]) {
        // Image was added to queue
        return 0.0;
    }
    if ([self isImageForKeyLoadingNow:key]) {
        INKURLRequest *request = [requestsInfo objectForKey:key];
        // Image downloading
        return request.downloadProgress;
    }
    UIImage *image = [_imagesInfo objectForKey:key];
    if (image) {
        // Image downloaded
        return 1.0;
    }
    return 0.0;
}

- (BOOL)isImageForKeyWaitingInQueue:(NSString *)key {
    INKURLRequest *request = [requestsInfo objectForKey:key];
    if (request) {
        return [requestsManager isRequestInQueue:request];
    }
    // Request was already processed, processing now or not added yet.
    return NO;
}

- (BOOL)isImageForKeyLoadingNow:(NSString *)key {
    INKURLRequest *request = [requestsInfo objectForKey:key];
    return [requestsManager isRequestIsProcessing:request];
}

- (BOOL)isImageForKeyWasDownloaded:(NSString *)key {
    UIImage *image = [_imagesInfo objectForKey:key];
    if (image) {
        return YES;
    }
    return NO;
}

- (BOOL)isImageForKeyWasAdded:(NSString *)key {
    UIImage *image = [_imagesInfo objectForKey:key];
    if (image) {
        // Downloaded already
        return YES;
    }
    if ([self isImageForKeyWaitingInQueue:key] || [self isImageForKeyLoadingNow:key]) {
        // In queue or downloading
        return YES;
    }
    // Never have been added
    return NO;
}

#pragma mark Images operations

- (UIImage*)imageForKey:(NSString *)key ifNotFindedDownloadFromLink:(NSString *)link {
    return [self imageForKey:key ifNotFindedDownloadFromLink:link withOperationBlock:nil];
}

- (UIImage*)imageForKey:(NSString *)key ifNotFindedDownloadFromLink:(NSString *)link 
     withOperationBlock:(INKImageLoaderOperationBlock)block {
    return [self imageForKey:key ifNotFindedDownloadFromLink:link withOperationBlock:block replaceAtFirstIndex:NO];
}

- (UIImage*)imageForKey:(NSString *)key ifNotFindedDownloadFromLink:(NSString *)link 
     withOperationBlock:(INKImageLoaderOperationBlock)block 
    replaceAtFirstIndex:(BOOL)replaceAtFirstIndex {
    UIImage *image = [_imagesInfo objectForKey:key];
    if (!image) {
        [self downloadImageFromLink:link forKey:key withOperationBlock:block important:replaceAtFirstIndex];
        return _defaultImage;
    }
    return image;
}

- (void)accelerateDownloadingImageForKey:(NSString *)key {
    INKURLRequest *request = [requestsInfo objectForKey:key];
    [requestsManager replaceRequestAtFirstPlace:request];
} 

- (void)downloadImageFromLink:(NSString *)link forKey:(NSString *)key {
    [self downloadImageFromLink:link forKey:key withOperationBlock:nil];
}

- (void)downloadImageFromLink:(NSString *)link forKey:(NSString *)key withOperationBlock:(INKImageLoaderOperationBlock)block {
    [self downloadImageFromLink:link forKey:key withOperationBlock:block important:NO];
}

- (void)downloadImportantImageFromLink:(NSString *)link forKey:(NSString *)key withOperationBlock:(INKImageLoaderOperationBlock)block {
    [self downloadImageFromLink:link forKey:key withOperationBlock:block important:YES];
}

#pragma mark Private methods 

- (void)downloadImageFromLink:(NSString *)link forKey:(NSString *)key withOperationBlock:(INKImageLoaderOperationBlock)block important:(BOOL)isImportant {
    // Check if downloading already
    INKURLRequest *request = [requestsInfo objectForKey:key];
    if (request) {
        // Move request as first index
        [self.requestsManager replaceRequestAtFirstPlace:request];
        return;
    }
        
    // Request is nil, we can continue
    NSURL *url = [NSURL URLWithString:link];
    request = [INKURLRequest requestWithURL:url];
    request.cachePolicy = self.requestsCachePolicy;
    request.timeoutInterval = self.requestsTimeout;
    [requestsInfo setObject:request forKey:key];
    INKURLRequestOperationBlock requestBlock = ^(BOOL finished, INKURLRequestOperationBlockCause cause) {
        float progress = [self downloadProgressOfImageForKey:key];
        NSData *data = request.incomingData;
        NSError *error = request.connectionError;
        if (finished) {
            UIImage *downloadedImage = [UIImage imageWithData:data];
            if (downloadedImage || progress==1.0) {
                [self addImage:downloadedImage withKey:key replace:YES];
            }
        }
        if (finished || cause==INKURLRequestOperationBlockCauseConnectionError) {
            [requestsInfo removeObjectForKey:key];
        }
        if (cause==INKURLRequestOperationBlockCauseConnectionError ||
            cause == INKURLRequestOperationBlockCauseDataPartLoaded ||
            cause == INKURLRequestOperationBlockCauseFinishLoading ||
            cause == INKURLRequestOperationBlockCauseStartLoading) {
            block(finished, data, key, progress, error);
        }
    };
    if (isImportant) {
        [requestsManager addImportantRequest:request withCompletionBlock:requestBlock];
    }
    else {
        [requestsManager addRequest:request withCompletionBlock:requestBlock];
    }
}

- (void)clearCache {
    [requestsManager cancelAllRequests];
    [requestsInfo removeAllObjects];
    [super clearCache];
}

@end
